#
# Copyright (C) 2023 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Helper for building observation ingestion manifest """
import logging
import sys
from pathlib import Path
from typing import List

from ingest_envoy.manifest_components import (
    PARAMETER_FILENAME,
    REIMAGING_FILENAME,
    TARFILE_EXT,
    WEBLOG_FILENAME,
    AncillaryProduct,
    OutputScienceProduct,
)
from ingest_envoy.utilities import AncillaryProductType, ScienceProductType

FITS = "fits"
IDIFITS = "idifits"
XML = "xml"
LOG = "log"
TEXT = "txt"
CSV = "csv"
TT0 = "tt0"
RMS = "rms"
PB = "pb"
MASK = "mask"
ALPHA = "alpha"
PBCOR = "pbcor"


# pylint: disable=R1721
class ObservationIngestionProductsFinder:
    """Finds ancillary science products and other ancillary products needed for observation ingestion"""

    def __init__(self, staging_source_dir: Path, sp_type: ScienceProductType):
        self.logger = logging.getLogger("ingest_envoy")
        self.staging_source_dir = staging_source_dir
        self.sp_type = sp_type
        self.files_found = [file for file in self.staging_source_dir.iterdir()]
        self.output_science_products = self._find_output_science_products()
        self.ancillary_products = self._find_ancillary_products()

    def _find_output_science_products(self) -> List[OutputScienceProduct]:
        """
        Find the ancillary products belonging to the science product*
        in the staging dir.

        N.B. ingestion artifacts file will not be there until manifest has been created!

        :return:
        """

        # Currently we only support ingestion of GMVA/VLBI observations. If there
        # is not yet implemented
        # is an SDM file present we know it's a different type of observation which
        for file in self.files_found:
            if file.name.endswith(XML):
                self.logger.error("Non-VLBA/GMVA observation ingestion is not currently implemented!")
                sys.exit(1)

        # Ingest all *fits files as science products
        fits_files = [file for file in self.files_found if file.name.endswith(IDIFITS)]
        self.logger.info(f"Science Product(s) to ingest: {fits_files}")

        science_products = []
        for file in fits_files:
            # Add all science products and their ancillaries
            science_products.append(
                OutputScienceProduct(
                    product_type=ScienceProductType.EXEC_BLOCK,
                    filename=file.name,
                )
            )

        return science_products

    def _find_ancillary_products(self) -> List[AncillaryProduct]:
        """
        Find the ancillary observation products in the staging dir: there should be a
        weblog and a pipeline artifacts tar. (The ingestion artifacts tar will be produced
        during the building of the manifest.)

        :return:
        """
        log_files = [file for file in self.files_found if file.name.endswith(LOG)]
        text_files = [file for file in self.files_found if file.name.endswith(TEXT)]
        self.logger.info(f"Ancillary product(s) to ingest: {[*log_files, *text_files]}")

        ancillary_products = []
        for file in log_files:
            ancillary_products.append(AncillaryProduct(type=AncillaryProductType.LOG_TYPE, filename=file.name))
        for file in text_files:
            ancillary_products.append(
                AncillaryProduct(type=AncillaryProductType.OBSERVATION_DETAILS, filename=file.name)
            )

        return ancillary_products
